﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Web;
using DynamicMVC.Annotations;
using DynamicMVC.Annotations.Enums;

namespace OrderEntryDemo.Models
{
    [DynamicEntity(IndexProperties = "OrderDate, CustomerId, OrderStatusId, Total", ShowEdit = false
        , CreateProperties = "OrderDate, CustomerId", InstanceIncludes = "OrderLines.Product,Customer,OrderStatus")]
    [DynamicMenuItem("Orders", "Order Entry")]
    [DynamicHeader(CreateHeader = "Select a Customer")]
    public class Order
    {
        public Order()
        {
            OrderLines = new HashSet<OrderLine>();
            OrderDate = DateTime.Now;
            OrderStatusId = 1; //new order
        }

        public int Id { get; set; }
        [DisplayName("Order Date")]
        [DisplayFormat(DataFormatString = "{0:d}")]
        [UIHint("DynamicEditorReadOnly")]
        public DateTime OrderDate { get; set; }
        public int CustomerId { get; set; }
        public Customer Customer { get; set; }
        [DisplayName("Order Status")]
        public int OrderStatusId { get; set; }
        public OrderStatus OrderStatus { get; set; }
        public decimal Total { get; set; }

        public ICollection<OrderLine> OrderLines { get; set; }

        [DynamicMethod(TemplateTypeEnum = TemplateTypeEnum.Details, InvokerName = "CalculateTotalMethodInvoker")]
        public void CalculateTotal()
        {
            foreach (var orderLine in OrderLines)
            {
                orderLine.Total = orderLine.Product.Price*orderLine.Quantity;
            }
            this.Total = OrderLines.Sum(x => x.Total);
        }
    }
}